/*
 * GREAT-ER
 *
 * DEF.H
 *
 * header file with definitions used in all modules
 *
 * author =  Geert Boeije
 *
 * changes by Intevation, http://intevation.net
 * state =   05-03-2003
*/

#ifndef DEF_H
#define DEF_H

/*
 *******************************************************************
 *
 * G E N E R A L    D E F I N I T I O N S
 *
 *******************************************************************
*/


/* version */

#define VERSION_NUMBER "1.0 <patch 4>"
#define VERSION_DATE   "05 MAR 2003"

/* testing the software ? */

/*
#define TEST_MODE
*/

#define EXPLANATION \
        "\nGREAT-ER Simulator\n" \
        "Version "VERSION_NUMBER" ("VERSION_DATE")\n\n" \
        "Geert Boeije\n" \
        "University of Gent, BIOMATH, Coupure 653, Gent, Belgium\n" \
        "tel. +32-9-264.61.79 - fax +32-9-223.49.41\n" \
        "email: geert.boeije@rug.ac.be\n\n\n" \
        "command line options:\n\n" \
        "simul \t-x <window left x coord> \tprogress window location X                  \n" \
        "\t-y <window top y coord> \tprogress window location Y\n" \
        "\t-t \"<window title>\"   \t\tprogress window title\n" \
        "\t-d \"<work directory>\"     \tsimulation directory name\n" \
        "\t-i \"<INI file name>\"      \tinitialization file name\n" \
        "\t-e \"<error file name>\"    \terror messages file name\n" \
        "\t-s <nr Monte Carlo shots> \tnumber of monte carlo samples\n" \
        "\t-h \t\t\tswitch to hide progress window\n" \
        "\t-m \t\t\tswitch to show control messages\n" \
        "\t-w \t\t\tswitch to hide error warning messages\n" \
        "\t-l \t\t\tswitch to create a log file\n" \
        "\t-? \t\t\tdisplay command line options\n"


/* GUI or text mode ??? */

#ifndef UNIX
  #ifdef __GNUC__
    #define GUI_MODE
  #endif
#endif


/*
 * ------------------------------------------------
 * GUI definitions
 * ------------------------------------------------
*/

/* default window positions */

#define DEFAULT_X        10
#define DEFAULT_Y        10

#define MAIN_WIN_WIDTH   250
#define MAIN_WIN_HEIGHT  150

#define DEFAULT_TITLE    "GREAT-ER Simulator"

/*
 * ------------------------------------------------
 * operational definitions
 * ------------------------------------------------
*/

#define INI_FILE                    "simul.ini"
#define DISTRI_TYPE_FILE            "distri.typ"

#define ERROR_LOG_FILE              "error.txt"
#define RESULTS_LOG_FILE            "results.txt"
#define OUTPUT_FILE                 "output.txt"

#define _UP_                        1
#define _DONT_KNOW_                 0

/* memory allocation */

#define MEMORY_ALLOC                calloc
#define FREE_MEMORY                 free


/*
 * -----------------------------------------------------------------
 * calculation accuracy typedefs
 * -----------------------------------------------------------------
*/

typedef double                      FloatType;
typedef FloatType*                  FloatPtr;

#define FLOAT_ID                    "lf"       /* used in fprintf and fscanf calls */
#define LITTLE                      1E-12
#define TINY                        1e-20


/*
 * -----------------------------------------------------------------
 * Monte Carlo
 * -----------------------------------------------------------------
*/

#define NR_DISTRITYPES              6

#define NO_DISTRIB                  0
#define NORMAL_DISTRIB              1
#define LOGNORMAL_DISTRIB           2
#define UNIFORM_DISTRIB             3
#define PECTYPE_DISTRIB             4
#define FLOWTYPE_DISTRIB            5

#define NR_PAR_NO                   1
#define NR_PAR_NORMAL               2
#define NR_PAR_LOGNORMAL            2
#define NR_PAR_UNIFORM              2
#define NR_PAR_PECTYPE              2
#define NR_PAR_FLOWTYPE             2

#define SUMMARY_DISTRI              (NR_NON_GEOREF_PARAMS + NrParamsPerSegment)

#define NR_SUMMARY_SUMS             2            /* nr. of sums of x^i to be memorized */

/*
 * -----------------------------------------------------------------
 * segment types
 * -----------------------------------------------------------------
*/

#define THE_SEA                     9
#define END_OF_TREE                 0
#define NOTHING_SPECIAL             1
#define CONFLUENCE                  2
#define BIFURCATION                -2

/*
 * -----------------------------------------------------------------
 * process classes
 *
 * indices for different model terminals
 * -----------------------------------------------------------------
*/

#define EMISSION                    0
#define TRANSTREAT                  1

/* emission */

#define EM                          0

/* trans-treat */

/* Intevation change: IN and OUT are standard makros of
 * some compilers and needed to be replaced. */
#define IN__                          0
#define OUT__                         1

/* summary variables */


/* OUT is already defined as 1 */

#define RIV_MAX                     0        /* summary state variables maximal value */
#define CAT_MAX                     3
#define RIV_OUT                     1        /* summary state variables output value */
#define CAT_OUT                     4
#define RIV_INT                     2        /* summary state variables internal value */
#define CAT_INT                     5

#define START_OF_DISCHARGE          CAT_MAX

#define NR_SUMMARY_TERMS            6        /* nr of terminals in summary variables */
#define NR_IFACE_TERMS              2        /* nr of terminals in interface variables (fixed at 2 ! ) */


#define ONE_STATEVAR_SIZE           (NR_EM_SUB + NR_TT_SUB * 2 + NR_IFACE_TERMS)


/*
 *******************************************************************
 *
 * M O D E L  -  S P E C I F I C     D E F I N I T I O N S
 *
 *******************************************************************
*/

/*
 * ---------------------------------------------
 * state variables
 * ---------------------------------------------
*/

/* used for stete variables and interface variables */

#define NR_STATEVARS                2

#define _Q_                         0        /* Q = flow (m3/s) */
#define _PHI_                       1        /* PHI = chemical mass flux (g/s) */

/* used for summary variables */

#define NR_SUMMARY_VARS             1

#define _C_                         0        /* C = chemical concentration (mg/l) or (g/m3) */

/*
 * ---------------------------------------------
 * processes
 * ---------------------------------------------
*/

#define NR_PROCESSES                9

#define DOM                         0
#define NONDOM                      1
#define RUNOFF                      2
#define ONSITE                      3
#define SEWER                       4
#define TREAT                       5
#define SURFWAT                     6
#define GROUNDWAT                   7
#define RIVER                       8

/* 'LAST' will be used as interface variables */

#define LAST                        RIVER

/* nr. of submodels in each process */

#define MAX_NR_SUBMODELS            5
                                    /* max nr. of submodels within a process */
#define MAX_NR_DESTINATIONS         5
                                    /* max nr. of dest. submods from a given process */

#define NR_SUB_DOM                  1
#define NR_SUB_NONDOM               1
#define NR_SUB_RUNOFF               1
#define NR_SUB_ONSITE               1
#define NR_SUB_SEWER                1
#define NR_SUB_TREAT                2
#define NR_SUB_SURFWAT              1
#define NR_SUB_GROUNDWAT            1
#define NR_SUB_RIVER                1

#define NR_EM_SUB                   (NR_SUB_DOM + NR_SUB_NONDOM + NR_SUB_RUNOFF)
#define NR_TT_SUB                   (NR_SUB_ONSITE + NR_SUB_SEWER + NR_SUB_TREAT \
                                     + NR_SUB_SURFWAT + NR_SUB_GROUNDWAT + NR_SUB_RIVER )
#define NR_SUB                      (NR_EM_SUB + NR_TT_SUB)

/* nr. of destinations from each process */

#define NR_DEST_DOM                 1
#define NR_DEST_NONDOM              1
#define NR_DEST_RUNOFF              1
#define NR_DEST_ONSITE              1
#define NR_DEST_SEWER               2
#define NR_DEST_TREAT               1
#define NR_DEST_SURFWAT             1
#define NR_DEST_GROUNDWAT           1
#define NR_DEST_RIVER               0

/* submodel codes */

#define DEFAULT                     0        
                                    /* used by processes with only one submodel */

/* domestic emission */

/* non-domestic emission */

/* domestic on-site treatment */

/* sewer */

#define COMBINED                    0

/* WWTP */

#define NONE                        0
#define TREATED                     1

/* some special definitions, sub-cases of WWTP */

/* NONE is already defined */
#define PRIMARY                     1
#define ACTIVATED_SLUDGE            2
#define TRICKLING_FILTER            3

#define NR_WWTP_CLASSES             1     /* nr of wwtps in one GU */

#endif

