/*
 * GREAT-ER
 *
 * DATA_IN.C
 *
 * data input section
 *
 * author =  Geert Boeije
 * state =   07.01.1999
*/

#include <stdio.h>
#include <string.h>
#include "init.h"
#include "names.h"
#include "error.h"
#include "usriface.h"
#include "navigate.h"
#include "data_in.h"

/*
 * ---------------------------------------------------------------------
 * open file for input
 * ---------------------------------------------------------------------
*/

int OpenInputFile ( FILE** source_arg,
                    char*  filename_arg,
                    int    err_arg )
{
  char f[1000];

  strcpy ( f, WorkDirectory );
  strcat ( f, filename_arg );

  /* open the file */

  if ( (*source_arg = fopen(f, "rt")) == NULL)
  {
    if (err_arg ) GenerateErrorMessage (101, f);
    return 0;
  }
  else
    return 1;
}

/*
 * ---------------------------------------------------------------------
 * read array from file
 *
 * if type_arg is 1, we will read a float array
 * otherwise, we will read an integer array
 * ---------------------------------------------------------------------
*/

long GetArrayFromFile ( void* array_arg,
                        FILE* source_arg,
                        int type_arg,
                        long nr_arg,
                        char*  filename_arg )
{
  FloatType read_float;
  FloatPtr float_array = (FloatPtr)array_arg;
  int read_int;
  int* int_array = (int*)array_arg;
  int i = 1;
  long counter = 0;
  char c;

  while (i != -1)    /* until end of file */
  {
    /* read a value */

    c = getc (source_arg);
    switch (c)
    {
      case '\n':
      case '\r':
      case ' ':
      case '\t':
        continue;

      case '/':
        c = getc(source_arg);
        if (c == '/')
        {
          while ( getc(source_arg) != '\n');
        }
        else
        {
          GenerateErrorMessage (105, filename_arg );
          return 0;
        }
          continue;
          
      default:
        ungetc(c, source_arg);
    }

    if ( type_arg == 1 )
      i = fscanf (source_arg, "%lf", &read_float);
    else
    {
      i = fscanf (source_arg, "%d", &read_int);
    }

    /* add the read value to parameter list - if relevant */

    if ( i != 1 && i != EOF )
    {
      GenerateErrorMessage (106, filename_arg );
      return 0;
    }


    if ( type_arg == 1 )
      float_array[ counter++ ] = read_float;
    else
      int_array[ counter++ ] = read_int;

    /* finished or not ?  (nr_arg = 0 means: go on until end of file) */
    
    if ( nr_arg && counter >= nr_arg )
      i = -1;
  }

  return counter;
}


/*
 * ---------------------------------------------------------------------
 * get SIMULATION CONTROL PARAMETERS from disk
 * ---------------------------------------------------------------------
*/

int ObtainControlParams ( FloatType* first_stretch_arg )
{
  FILE* the_file;
  FloatType the_first;
  int test = 1;

  /* from INI FILE */

  if (!OpenInputFile ( &the_file, INIFileName, 1 ))
    return 0;

  fscanf ( the_file, "\n" );

  test = test && fscanf ( the_file, "LOG CONTROL PARAMS = %d\n", &LogCtrlParams );
  test = test && fscanf ( the_file, "LOG SHOT PARAMETERS = %d\n", &LogShotParameters );
  test = test && fscanf ( the_file, "LOG SHOT RESULTS = %d\n", &LogShotResults );
  test = test && fscanf ( the_file, "LOG SUMMARY RESULTS =%d\n", &LogSummaryResults );

  test = test && fscanf ( the_file, "NON-GEOREF FILE = %s\n", NonGeoRefFileName );
  test = test && fscanf ( the_file, "CHEMICAL FILE = %s\n", ChemicalFileName );
  test = test && fscanf ( the_file, "RIVER FILE = %s\n", RiverFileName );
  test = test && fscanf ( the_file, "RIVER CLASS FILE = %s\n", RivClassFileName );
  test = test && fscanf ( the_file, "DISCHARGE FILE = %s\n", DischargeFileName );
  test = test && fscanf ( the_file, "WWTP FILE = %s\n", WWTPFileName );
  test = test && fscanf ( the_file, "EMISSION FILE = %s\n", EmissionFileName );

  test = test && fscanf ( the_file, "WWTP EXCEPTIONS FILE = %s\n", WWTPExceptionsFileName );
  test = test && fscanf ( the_file, "RIVER EXCEPTIONS FILE = %s\n", RiverExceptionsFileName );

  test = test && fscanf ( the_file, "DISTRIBUTION TYPE FILE = %s\n", DistriFileName );
  test = test && fscanf ( the_file, "OUTPUT FILE = %s\n", OutputFileName );
  test = test && fscanf ( the_file, "RESULTS FILE = %s\n", ResultsFileName );

  test = test && fscanf ( the_file, "USE SITE-SPECIFIC EXCEPTIONS = %d\n", &UseExceptions );
  test = test && fscanf ( the_file, "DEFAULT COMPLEXITY MODE RIVER = %d\n", &ComplexityModeRiver );
  test = test && fscanf ( the_file, "DEFAULT COMPLEXITY MODE WWTP = %d\n", &ComplexityModeWWTP );
  test = test && fscanf ( the_file, "DEFAULT COMPLEXITY MODE SEWER = %d\n", &ComplexityModeSewer );
  test = test && fscanf ( the_file, "IMPOSE DEFAULT COMPLEXITY MODE RIVER = %d\n", &ImposeComplexityModeRiver );  
  test = test && fscanf ( the_file, "IMPOSE DEFAULT COMPLEXITY MODE WWTP = %d\n", &ImposeComplexityModeWWTP );  
  test = test && fscanf ( the_file, "IMPOSE DEFAULT COMPLEXITY MODE SEWER = %d\n", &ImposeComplexityModeSewer );  

  if (! fscanf ( the_file, "FIRST SEGMENT = %lf\n", first_stretch_arg ) )
    *first_stretch_arg = -1;

  fclose ( the_file );

  if (!test)
  {
    GenerateErrorMessage (104, INIFileName);
    return 0;
  }

  /* from RIVER FILE */

  if (!OpenInputFile ( &the_file, RiverFileName,1 ))
    return 0;

  test = GetArrayFromFile ( &NrSegments, the_file, 0, 1, RiverFileName );
  test = test & GetArrayFromFile ( &the_first, the_file, 1, 1, RiverFileName );

  if ( *first_stretch_arg == -1 )
    *first_stretch_arg = the_first;

  fclose ( the_file );

  /* defaults + checking */

  if ( DistriFileName[0] == '-' ) strcpy (DistriFileName, DISTRI_TYPE_FILE);
  if ( OutputFileName[0] == '-' ) strcpy (OutputFileName, OUTPUT_FILE);
  if ( ResultsFileName[0] == '-' ) strcpy (ResultsFileName, RESULTS_LOG_FILE);

  if ( ( ComplexityModeRiver < 1 || ComplexityModeRiver > 3) && (ComplexityModeRiver != -1) )
  {
    GenerateErrorMessage (903, &ComplexityModeRiver);
    ComplexityModeRiver = 1;
  }

  if ( ( ComplexityModeWWTP < 1 || ComplexityModeWWTP > 3) && (ComplexityModeWWTP != -1) )
  {
    GenerateErrorMessage (904, &ComplexityModeWWTP);
    ComplexityModeWWTP = 1;
  }

  if ( ( ComplexityModeSewer < 1 || ComplexityModeSewer > 3) && (ComplexityModeSewer != -1) )
  {
    GenerateErrorMessage (905, &ComplexityModeSewer);
    ComplexityModeSewer = 1;
  }

  if ( ( ImposeComplexityModeRiver != 1 ) && ( ImposeComplexityModeRiver != 0 ) )
  {
    GenerateErrorMessage (910, &ImposeComplexityModeRiver);
    ImposeComplexityModeRiver = 0;
  }

  if ( ( ImposeComplexityModeWWTP != 1 ) && ( ImposeComplexityModeWWTP != 0 ) )
  {
    GenerateErrorMessage (911, &ImposeComplexityModeWWTP);
    ImposeComplexityModeWWTP = 0;
  }

  if ( ( ImposeComplexityModeSewer != 1 ) && ( ImposeComplexityModeSewer != 0 ) )
  {
    GenerateErrorMessage (912, &ImposeComplexityModeSewer);
    ImposeComplexityModeSewer = 0;
  }

  if ( NrSegments <= 0 || !test )
  {
    GenerateErrorMessage (102,NULL);
    return 0;
  }

  /* from DISCHARGE FILE */

  if (!OpenInputFile ( &the_file, DischargeFileName,1 ))
    return 0;

  test = GetArrayFromFile ( &NrSegmentsDischarge, the_file, 0, 1, DischargeFileName );

  fclose ( the_file );

  if ( NrSegmentsDischarge < 0 )
    NrSegmentsDischarge = 0;

  NrSegmentsOnlyRiver = NrSegments - NrSegmentsDischarge;

  /* from EMISSION FILE */

  if (!OpenInputFile ( &the_file, EmissionFileName,1 ))
    return 0;

  test = GetArrayFromFile ( &NrEmissions, the_file, 0, 1, EmissionFileName );

  fclose ( the_file );

  if ( NrEmissions < 0 )
    NrEmissions = 0;

  /* from WWTP FILE */

  if (!OpenInputFile ( &the_file, WWTPFileName,1 ))
    return 0;

  test = GetArrayFromFile ( &NrWWTPs, the_file, 0, 1, WWTPFileName );

  fclose ( the_file );

  if ( NrWWTPs < 0 )
    NrWWTPs = 0;

  /* from EXCEPTIONS files */

  if ( UseExceptions )
  {
    if (!OpenInputFile ( &the_file, WWTPExceptionsFileName, 0 ))
      NrWWTPExceptions = 0;
    else
    {
      test = GetArrayFromFile ( &NrWWTPExceptions, the_file, 
                                0, 1, WWTPExceptionsFileName );

      fclose ( the_file );
    }

    if (!OpenInputFile ( &the_file, RiverExceptionsFileName, 0 ))
      NrRiverExceptions = 0;
    else
    {
      test = GetArrayFromFile ( &NrRiverExceptions, the_file, 
                                0, 1, RiverExceptionsFileName );
  
      fclose ( the_file );
    }
  }
  else
  {
    NrWWTPExceptions = 0;    
    NrRiverExceptions = 0;
  }

  /* from RIVCLASS FILE */

  if (!OpenInputFile ( &the_file, RivClassFileName, 1 ))
    return 0;

  test = GetArrayFromFile ( &NrRivClasses, the_file, 0, 1, RivClassFileName );

  fclose ( the_file );

  if ( NrRivClasses <= 0 || !test )
  {
    GenerateErrorMessage (107,NULL);
    return 0;
  }

  return 1;
}

/*
 * ---------------------------------------------------------------------
 * get PARAMETER DISTRIBUTION TYPES from disk
 * ---------------------------------------------------------------------
*/

int ObtainDistriTypes ( void )
{
  FILE* distri_file;
  int test = 1;
  int nr;

  /* open the distribution types file */

  if (!OpenInputFile ( &distri_file, DistriFileName, 1 ))
    return 0;

  fscanf ( distri_file, "\n" );

  /* non-georeferenced parameters */

  nr = NR_NON_GEOREF_PARAMS;
  test = test && ( GetArrayFromFile ( NonGeoRefDistriType, 
                                      distri_file, 
                                       0, (long)nr, DISTRI_TYPE_FILE ) == nr );

  /* chemical parameters */

  nr = NR_CHEMICAL_PARAMS;
  test = test && ( GetArrayFromFile ( ChemicalDistriType, 
                                      distri_file, 
                                       0, (long)nr, DISTRI_TYPE_FILE ) == nr );


  /* river parameters */

  nr = NrParamsOnlyRiver;
  test = test && ( GetArrayFromFile ( RiverDistriType, 
                                      distri_file, 
                                      0, (long)nr, DISTRI_TYPE_FILE ) == nr );

  /* discharge parameters */

  nr = NrParamsDischarge - NrParamsOnlyRiver;
  test = test && ( GetArrayFromFile ( DischargeDistriType, 
                                      distri_file, 
                                      0, (long)nr, DISTRI_TYPE_FILE ) == nr );

  /* emission data */

  nr = NR_EMISSION_PARAMS;
  test = test && ( GetArrayFromFile ( EmissionDistriType, 
                                      distri_file, 
                                      0, (long)nr, DISTRI_TYPE_FILE ) == nr );

  /* WWTP data */

  nr = NR_WWTP_PARAMS;
  test = test && ( GetArrayFromFile ( WWTPDistriType, 
                                      distri_file, 
                                      0, (long)nr, DISTRI_TYPE_FILE ) == nr );

  /* River class data */

  nr = NR_RIVCLASS_PARAMS;
  test = test && ( GetArrayFromFile ( RivClassDistriType, 
                                      distri_file, 
                                      0, (long)nr, DISTRI_TYPE_FILE ) == nr );
 
  /* results distribution types */

  nr = NR_SUMMARY_VARS * NR_SUMMARY_TERMS;
  test = test && ( GetArrayFromFile ( SummaryDistriType, 
                                      distri_file, 
                                      0, (long)nr, DISTRI_TYPE_FILE ) == nr );

  /* finished */

  fclose ( distri_file );

  if (!test)
  {
    GenerateErrorMessage ( 103, DISTRI_TYPE_FILE );
    return 0; 
  }

  return 1;
}

/*
 * ---------------------------------------------------------------------
 * get MODEL INPUT DATA from disk
 * ---------------------------------------------------------------------
*/

int ObtainInputData ( void )
{
  FILE* data_file;
  FloatType dummy[2];
  long nr;
  int test;

  /* read the NON-GEOREFERENCED DATA file / name = NonGeoRefFileName */

  if (!OpenInputFile ( &data_file, NonGeoRefFileName, 1 ))
    return 0;

  /* data set contains <TotNonGeoRefDistriPar> data values */

  nr = (long)TotNonGeoRefDistriPar;
  test = ( GetArrayFromFile ( Distri_NonGeoRefParams, 
                              data_file, 
                              1, nr, NonGeoRefFileName ) == nr );
  fclose ( data_file );

  if (!test)
  {
    GenerateErrorMessage ( 103, NonGeoRefFileName );
    return 0; 
  }

  /* read the CHEMICAL DATA file / name = ChemicalFileName */

  if (!OpenInputFile ( &data_file, ChemicalFileName, 1 ))
    return 0;

  /* data set contains <TotChemicalDistriPar> data values */

  nr = (long)TotChemicalDistriPar;
  test = ( GetArrayFromFile ( Distri_ChemicalParams, 
                              data_file, 
                              1, nr, ChemicalFileName ) == nr );
  fclose ( data_file );

  if (!test)
  {
    GenerateErrorMessage ( 103, ChemicalFileName );
    return 0; 
  }


  /* read the RIVER file / name = RiverFileName */

  if (!OpenInputFile ( &data_file, RiverFileName, 1 ))
    return 0;

  /* skip value 1 and 2 (nr segs and first seg) */

  test = ( GetArrayFromFile ( dummy, data_file, 1, 2, RiverFileName ) == 2);

  /* data set contains <NrSegments> segments */
  /* each segment contains <TotRiverDistriPar> data values */

  nr = (long)NrSegments * (long)TotRiverDistriPar;
  test = test && ( GetArrayFromFile ( Distri_RiverParams, 
                                      data_file, 
                                      1, nr, RiverFileName ) == nr );
  fclose ( data_file );

  if (!test)
  {
    GenerateErrorMessage ( 103, RiverFileName );
    return 0; 
  }

  if ( NrSegmentsDischarge )
  {
    /* read the DISCHARGE file / name = DischargeFileName */

    if (!OpenInputFile ( &data_file, DischargeFileName, 1 ))
      return 0;

    /* skip first number = Nr. discharges */

    test = ( GetArrayFromFile ( dummy, data_file, 1, 1, DischargeFileName ) == 1 );

    /* data set contains <NrSegmentsDischarge> segments */
    /* each segment contains <TotDischargeDistriPar> data values */

    nr = (long)NrSegmentsDischarge * (long)TotDischargeDistriPar;

    test = test && ( GetArrayFromFile ( Distri_DischargeParams, 
                                        data_file, 
                                        1, nr, DischargeFileName ) == nr );
    fclose ( data_file );

    if (!test)
    {
      GenerateErrorMessage ( 103, DischargeFileName );
      return 0;
    }
  }

  if ( NrRivClasses )
  {
    /* read the RIV CLASS file / name = RivClassFileName */

    if (!OpenInputFile ( &data_file, RivClassFileName, 1 ))
      return 0;

    /* skip first number = Nr. riv class data points */

    test = ( GetArrayFromFile ( dummy, data_file, 1, 1, RivClassFileName ) == 1 );

    nr = (long)NrRivClasses * (long)TotRivClassDistriPar;
    test = test && ( GetArrayFromFile ( Distri_RivClassParams, 
                                        data_file, 
                                        1, nr, RivClassFileName ) == nr );

    fclose ( data_file );

    if (!test)
    {
      GenerateErrorMessage ( 103, RivClassFileName );
      return 0;
    }
  }

  if ( NrWWTPs )
  {
    /* read the WWTP file / name = WWTPFileName */

    if (!OpenInputFile ( &data_file, WWTPFileName, 1 ))
      return 0;

    /* skip first number = Nr. WWTP data points */

    test = ( GetArrayFromFile ( dummy, data_file, 1, 1, WWTPFileName ) == 1 );

    nr = (long)NrWWTPs * (long)TotWWTPDistriPar;
    test = test && ( GetArrayFromFile ( Distri_WWTPParams, 
                                        data_file, 
                                        1, nr, WWTPFileName ) == nr );
    fclose ( data_file );

    if (!test)
    {
      GenerateErrorMessage ( 103, WWTPFileName );
      return 0;
    }
  }

  if ( NrEmissions )
  {
    /* read the EMISSION file / name = EmissionFileName */

    if (!OpenInputFile ( &data_file, EmissionFileName, 1 ))
      return 0;

    /* skip first number = Nr. market data points */

    test = ( GetArrayFromFile ( dummy, data_file, 1, 1, EmissionFileName ) == 1 );

    /* data set contains <NrEmissions> segments */
    /* each segment contains <TotEmissionDistriPar> data values */

    nr = (long)NrEmissions * (long)TotEmissionDistriPar;
    test = test && ( GetArrayFromFile ( Distri_EmissionParams, 
                                        data_file, 
                                        1, nr, EmissionFileName ) == nr );
    fclose ( data_file );

    if (!test)
    {
      GenerateErrorMessage ( 103, EmissionFileName );
      return 0;
    }
  }


  return 1;
}

/*
 * ---------------------------------------------------------------------
 * get EXCEPTIONS data from disk
 * ---------------------------------------------------------------------
*/

int ObtainExceptions ( void )
{
  FILE* exceptions_file;

  int test = 1;
  int nr;

  if (!UseExceptions) return 1;

  /* open + read the river exceptions file - if any */

  if ( ! OpenInputFile ( &exceptions_file, RiverExceptionsFileName, 0 ) )
    NrRiverExceptions = 0;
  else
  {
    test = GetArrayFromFile ( &nr, exceptions_file, 0, 1, RiverExceptionsFileName );

    test = test && ( GetArrayFromFile ( RiverExceptions, 
                                        exceptions_file, 
                                        1, 
                                        NrRiverExceptions * 2,
                                        RiverExceptionsFileName ) == NrRiverExceptions*2 );

    fclose ( exceptions_file );

    if (!test)
    {
      GenerateErrorMessage ( 103, RiverExceptionsFileName );
      return 0;
    }
  }

  /* open + read the WWTP exceptions file - if any */

  if (!OpenInputFile ( &exceptions_file, WWTPExceptionsFileName, 0 ))
    NrWWTPExceptions = 0;
  else
  {
    test = GetArrayFromFile ( &nr, exceptions_file, 0, 1, WWTPExceptionsFileName );

    test = test && ( GetArrayFromFile ( WWTPExceptions, 
                                        exceptions_file, 
                                        1,
                                        NrWWTPExceptions * 2,
                                        WWTPExceptionsFileName ) == NrWWTPExceptions*2 );

    fclose ( exceptions_file );

    if (!test)
    {
      GenerateErrorMessage ( 103, WWTPExceptionsFileName );
      return 0;
    }
  }

  return 1;
}

