/*
 * GREAT-ER
 *
 * INIT.C
 *
 * author =  Geert Boeije
 * state =   07.01.1999
*/

#include "init.h"

/* global variables */

char SimulationTitle[150] = DEFAULT_TITLE;
char WorkDirectory[500] = "";

int FloatSize;
int IntSize;

int NrEmissionSubmodels;
int NrTransTreatSubmodels;
int NrSubmodels;

int ModelClass        [NR_PROCESSES];
int NrSub             [NR_PROCESSES];
int NrDestinations    [NR_PROCESSES];
int Destination       [NR_PROCESSES][MAX_NR_DESTINATIONS][2];
int DestNrBypass      [NR_PROCESSES];
int NrParamsInProcess [NR_PROCESSES];
int NrParamsOnlyRiver;
int NrParamsDischarge;

int OneStateVarSize;
int AllStateVarSize;
int AmatrixSize;
int AsubMatrixSize;

long AllIfaceSize;
long AllParamSize;
long AllSummarySize;

int NrSegments;
int NrSegmentsOnlyRiver;
int NrSegmentsDischarge;
int NrRivClasses;
int NrEmissions;
int NrWWTPs;

int NrWWTPExceptions;
int NrRiverExceptions;

int* SegmentDone;

int* SegmentParOffset;
int  ParametersArrayOffset [NR_PROCESSES + 1];
int  StateVarsArrayOffset  [NR_PROCESSES + 1];
int* AsubMatrixOffset;

int NrDistriParams [NR_DISTRITYPES];
int NrMonteCarloShots;
int* NonGeoRefDistriParIndex;
int* NonGeoRefDistriType;
int* ChemicalDistriParIndex;
int* ChemicalDistriType;
int* RiverDistriParIndex;
int* RiverDistriType;
int* DischargeDistriParIndex;
int* DischargeDistriType;
int* EmissionDistriType;
int* EmissionDistriParIndex;
int* RivClassDistriType;
int* RivClassDistriParIndex;
int* WWTPDistriType;
int* WWTPDistriParIndex;
int* SummaryDistriParIndex;
int* SummaryDistriType;
int TotNonGeoRefDistriPar;
int TotChemicalDistriPar;
int TotDischargeDistriPar;
int TotRiverDistriPar;
int TotSummaryDistriPar;
int TotEmissionDistriPar;
int TotWWTPDistriPar;
int TotRivClassDistriPar;

int WWTP_ID_Location;
int Emission_ID_Location;

NonGeoRef_ParPtr NonGeoRefParams;
Chemical_ParPtr ChemicalParams;
FloatPtr Distri_NonGeoRefParams;
FloatPtr Distri_ChemicalParams;
FloatPtr Distri_RiverParams;
FloatPtr Distri_DischargeParams;
FloatPtr Distri_EmissionParams;
FloatPtr Distri_RivClassParams;
FloatPtr Distri_WWTPParams;
FloatPtr EmissionData;
FloatPtr WWTPData;
FloatPtr CSewer_Q_DWF;
FloatPtr Distri_SummaryVars;
FloatPtr WWTPExceptions;
FloatPtr RiverExceptions;
Map_Ptr  Map;

FloatPtr A;
FloatPtr B;
FloatPtr X;

FILE* OutputFile;
FILE* ErrorFile;
FILE* LogFile;

char NonGeoRefFileName[500];
char ChemicalFileName[500];
char RiverFileName[500];
char DischargeFileName[500];
char EmissionFileName[500];
char WWTPFileName[500];
char RivClassFileName[500];
char DistriFileName[500];
char OutputFileName[500];
char ResultsFileName[500];
char ErrorFileName[500] = ERROR_LOG_FILE;
char INIFileName[500] = INI_FILE;
char WWTPExceptionsFileName[500];
char RiverExceptionsFileName[500];

int LogCtrlParams;
int LogShotParameters;
int LogShotResults;
int LogSummaryResults;

int LogToFile;
int ShowMessages;
int ShowErrorMessages;
int Visible;

int ComplexityModeRiver;
int ComplexityModeWWTP;
int ComplexityModeSewer;
int ImposeComplexityModeRiver;
int ImposeComplexityModeWWTP;
int ImposeComplexityModeSewer;
int UseExceptions;

/* success or not ? */

int ErrorFree = 1;
int AllFinished = 0;


char C[100];

/***************************************************************************************/
/***************************************************************************************/
/***************************************************************************************/
#ifdef TEST_MODE
FILE* TestLog;
#endif  
/***************************************************************************************/
/***************************************************************************************/
/***************************************************************************************/



/* initialization routine */

void Initialize (void)
{
  int i, counter=0;

  /*
   * ------------------------------------------------
   * general
   * ------------------------------------------------
  */

  /* Operational */

  FloatSize = sizeof (FloatType);
  IntSize   = sizeof (int);

  /* Monte Carlo */

  NrDistriParams[NO_DISTRIB]         = NR_PAR_NO;
  NrDistriParams[NORMAL_DISTRIB]     = NR_PAR_NORMAL;
  NrDistriParams[LOGNORMAL_DISTRIB]  = NR_PAR_LOGNORMAL;
  NrDistriParams[UNIFORM_DISTRIB]    = NR_PAR_UNIFORM;
  NrDistriParams[PECTYPE_DISTRIB]    = NR_PAR_PECTYPE;
  NrDistriParams[FLOWTYPE_DISTRIB]   = NR_PAR_FLOWTYPE;

  /* Submodels */

  NrEmissionSubmodels    = NR_EM_SUB;
  NrTransTreatSubmodels  = NR_TT_SUB;
  NrSubmodels            = NR_SUB;

  /* state variables array sizes */

  OneStateVarSize = ONE_STATEVAR_SIZE;
  AllStateVarSize = OneStateVarSize * NR_STATEVARS;

  /*
   * ------------------------------------------------
   * processes
   * ------------------------------------------------
  */

  ModelClass[DOM]       = EMISSION;
  ModelClass[NONDOM]    = EMISSION;
  ModelClass[RUNOFF]    = EMISSION;
  ModelClass[ONSITE]    = TRANSTREAT;
  ModelClass[SEWER]     = TRANSTREAT;
  ModelClass[TREAT]     = TRANSTREAT;
  ModelClass[GROUNDWAT] = TRANSTREAT;
  ModelClass[SURFWAT]   = TRANSTREAT;
  ModelClass[RIVER]     = TRANSTREAT;

  /*
   * ------------------------------------------------
   * submodels
   * ------------------------------------------------
  */

  /* number of submodels in each process */

  NrSub[DOM]       = NR_SUB_DOM;
  NrSub[NONDOM]    = NR_SUB_NONDOM;
  NrSub[RUNOFF]    = NR_SUB_RUNOFF;
  NrSub[ONSITE]    = NR_SUB_ONSITE;
  NrSub[SEWER]     = NR_SUB_SEWER;
  NrSub[TREAT]     = NR_SUB_TREAT;
  NrSub[GROUNDWAT] = NR_SUB_GROUNDWAT;
  NrSub[SURFWAT]   = NR_SUB_SURFWAT;
  NrSub[RIVER]     = NR_SUB_RIVER;

  /* Architecture of Submodels interconnection */

  /* nr. of Destination submodels from each process  */

  NrDestinations[DOM]       = NR_DEST_DOM;
  NrDestinations[NONDOM]    = NR_DEST_NONDOM;
  NrDestinations[RUNOFF]    = NR_DEST_RUNOFF;
  NrDestinations[ONSITE]    = NR_DEST_ONSITE;
  NrDestinations[SEWER]     = NR_DEST_SEWER;
  NrDestinations[TREAT]     = NR_DEST_TREAT;
  NrDestinations[GROUNDWAT] = NR_DEST_GROUNDWAT;
  NrDestinations[SURFWAT]   = NR_DEST_SURFWAT;
  NrDestinations[RIVER]     = NR_DEST_RIVER;

  /* list of Destinations coming from each process */

  Destination[DOM][0][0]       = ONSITE;      Destination[DOM][0][1]       = DEFAULT;

  Destination[NONDOM][0][0]    = SEWER;       Destination[NONDOM][0][1]    = DEFAULT;

  Destination[RUNOFF][0][0]    = SEWER;       Destination[RUNOFF][0][1]    = DEFAULT;

  Destination[ONSITE][0][0]    = SEWER;       Destination[ONSITE][0][1]    = DEFAULT;

  Destination[SEWER][0][0]     = TREAT;       Destination[SEWER][0][1]     = NONE;
  Destination[SEWER][1][0]     = TREAT;       Destination[SEWER][1][1]     = TREATED;

  Destination[TREAT][0][0]     = SURFWAT;     Destination[TREAT][0][1]     = DEFAULT;

  Destination[GROUNDWAT][0][0] = SURFWAT;     Destination[GROUNDWAT][0][1] = DEFAULT;

  Destination[SURFWAT][0][0]   = RIVER;       Destination[SURFWAT][0][1]   = DEFAULT;

  /* which is the 'link' to the bypass process (ie. groundwater leaks) for each process ?
     -1 means there is no link to the bypass process */

  DestNrBypass[DOM]        = -1;
  DestNrBypass[NONDOM]     = -1;
  DestNrBypass[RUNOFF]     = -1;
  DestNrBypass[ONSITE]     = -1;
  DestNrBypass[SEWER]      = -1;
  DestNrBypass[TREAT]      = -1;
  DestNrBypass[GROUNDWAT]  = -1;
  DestNrBypass[SURFWAT]    = -1;
  DestNrBypass[RIVER]      = -1;

  /*
   * -----------------------------------------------------------------
   * Parameter Structures
   * -----------------------------------------------------------------
   */

  /* number of parameters in each process */

  NrParamsInProcess[DOM] =       NR_DOM_PARAMS;
  NrParamsInProcess[NONDOM] =    NR_NONDOM_PARAMS;
  NrParamsInProcess[RUNOFF] =    NR_RUNOFF_PARAMS;
  NrParamsInProcess[ONSITE] =    NR_ONSITE_PARAMS;
  NrParamsInProcess[SEWER] =     NR_SEWER_PARAMS;
  NrParamsInProcess[TREAT] =     NR_TREAT_PARAMS;
  NrParamsInProcess[GROUNDWAT] = NR_GROUNDWAT_PARAMS;
  NrParamsInProcess[SURFWAT] =   NR_SURFWAT_PARAMS;
  NrParamsInProcess[RIVER] =     NR_RIVER_PARAMS;
    
  /* Number of parameters in 'ONLY RIVER' segment */
  
  NrParamsOnlyRiver = NR_GEOREF_GEN_PARAMS + NrParamsInProcess[RIVER];
  
  /* Number of parameters in 'RIVER + DISCHARGE' segment */
  
  NrParamsDischarge =   NR_GEOREF_GEN_PARAMS + 1; /* +1 is for emission ID ! */
  for ( i=0 ; i < NR_PROCESSES ; ++i )
    NrParamsDischarge += NrParamsInProcess[i];

  /* locate where the WWTP and emission numbers are situated in the Distri_WWTPParams array
     temporary values, to be corrected later on in navigate.c */

  for ( i=0, counter=0 ; i<TREAT ; ++i )
    counter += NrParamsInProcess[i];

  WWTP_ID_Location = counter + 1; /* +1 is for emission ID ! */


  /* emission ID is first data point in discharge data set */

  Emission_ID_Location = 0;
}


