/*
 * GREAT-ER
 *
 * MC-CHECK.C
 *
 * Monte Carlo - checking whether a shot is OK
 *
 * author =  Geert Boeije
 * state =   02.10.1998
*/

#include "init.h"
#include "navigate.h"
#include "mc-check.h"

FloatType BetweenOneZero ( FloatType value_arg )
{
  if ( value_arg < 0 )
    return 0;

  if ( value_arg > 1 )
    return 1;

  return value_arg;
}

FloatType MinusOneToOne ( FloatType value_arg )
{
  if ( value_arg < -1 )
    return -1;

  if ( value_arg > 1 )
    return 1;

  return value_arg;
}

FloatType OneOrZero ( FloatType value_arg )
{
  if ( value_arg <= 0 )
    return 0;

  if ( value_arg > 1 )
    return 1;  
  return value_arg;
}

FloatType NotNegative ( FloatType value_arg )
{
  if ( value_arg < 0 )
    return 0;
  else
    return value_arg;
}

FloatType MinusOneOrNotNegative ( FloatType value_arg )
{
  if ( value_arg == -1 )
    return -1;
  else
    return NotNegative ( value_arg );
}

FloatType AboveZero ( FloatType value_arg )
{
  if ( value_arg < LITTLE )
    return LITTLE;
  else
    return value_arg;
}

void CheckNonGeoRefShot ( void )
{
  NonGeoRefParams->groundwater_conservative 
    = OneOrZero (NonGeoRefParams->groundwater_conservative);
  /* water or sewage cannot freeze in our model concept */
  NonGeoRefParams->temp_water = AboveZero (NonGeoRefParams->temp_water);
  NonGeoRefParams->temp_sewage = AboveZero (NonGeoRefParams->temp_sewage);
  /* the air can do what it wants to *//*NonGeoRefParams->temp_air*/
  NonGeoRefParams->wind_speed = NotNegative (NonGeoRefParams->wind_speed);
  NonGeoRefParams->h_air = AboveZero (NonGeoRefParams->h_air);
  NonGeoRefParams->SS_sewage = AboveZero (NonGeoRefParams->SS_sewage);
  NonGeoRefParams->f_oc_sewage = AboveZero (NonGeoRefParams->f_oc_sewage);
  NonGeoRefParams->ro_SS_sewage = AboveZero (NonGeoRefParams->ro_SS_sewage);
  NonGeoRefParams->PHI_cap_BOD = AboveZero (NonGeoRefParams->PHI_cap_BOD);
  NonGeoRefParams->PHI_cap_N = AboveZero (NonGeoRefParams->PHI_cap_N);
  NonGeoRefParams->R_prim_BOD= NotNegative (NonGeoRefParams->R_prim_BOD);
  NonGeoRefParams->R_prim_N = NotNegative (NonGeoRefParams->R_prim_N);
  NonGeoRefParams->r_end = AboveZero (NonGeoRefParams->r_end);
  NonGeoRefParams->f_oc_ML = AboveZero (NonGeoRefParams->f_oc_ML);
  NonGeoRefParams->ro_SS_ML = AboveZero (NonGeoRefParams->ro_SS_ML);
  NonGeoRefParams->PSI = AboveZero (NonGeoRefParams->PSI);
  NonGeoRefParams->kGa_kLa = AboveZero (NonGeoRefParams->kGa_kLa);
  NonGeoRefParams->ML_salinity = NotNegative (NonGeoRefParams->ML_salinity);
  /* NonGeoRefParams->CS_Q_DWF */
  NonGeoRefParams->K_air = AboveZero (NonGeoRefParams->K_air);
  NonGeoRefParams->K_water = AboveZero (NonGeoRefParams->K_water);
  NonGeoRefParams->k_sorb_prim = AboveZero (NonGeoRefParams->k_sorb_prim);
  NonGeoRefParams->k_sorb_ML = AboveZero (NonGeoRefParams->k_sorb_ML);
  NonGeoRefParams->k_sorb_sec = AboveZero (NonGeoRefParams->k_sorb_sec);
  NonGeoRefParams->a_prim = NotNegative (NonGeoRefParams->a_prim);
  NonGeoRefParams->a_sec = NotNegative (NonGeoRefParams->a_sec);

  NonGeoRefParams->temp_corr = MinusOneToOne (NonGeoRefParams->temp_corr);
  NonGeoRefParams->wind_corr = MinusOneToOne (NonGeoRefParams->wind_corr);
  NonGeoRefParams->SS_corr = MinusOneToOne (NonGeoRefParams->SS_corr);
  NonGeoRefParams->DO_corr = MinusOneToOne (NonGeoRefParams->DO_corr);
  NonGeoRefParams->BOD_corr = MinusOneToOne (NonGeoRefParams->BOD_corr);
  NonGeoRefParams->k_SS_corr = MinusOneToOne (NonGeoRefParams->k_SS_corr);
}


void CheckChemicalShot ( void )
{
  ChemicalParams->MM = AboveZero (ChemicalParams->MM);
  ChemicalParams->K_ow = NotNegative (ChemicalParams->K_ow);
  ChemicalParams->P_vapour = NotNegative (ChemicalParams->P_vapour);
  ChemicalParams->solubility = AboveZero (ChemicalParams->solubility);

  /* ChemicalParams->pKa  correct: pKa can be negative */

  if ( ChemicalParams->a < -LITTLE ) ChemicalParams->a = -1;
  else if ( ChemicalParams->a > LITTLE ) ChemicalParams->a = 1;
  else ChemicalParams->a = 0;

  ChemicalParams->Kd_river = MinusOneOrNotNegative (ChemicalParams->Kd_river);
  ChemicalParams->Kd_sewage = MinusOneOrNotNegative (ChemicalParams->Kd_sewage);
  ChemicalParams->Kd_ML = MinusOneOrNotNegative (ChemicalParams->Kd_ML);
  ChemicalParams->H = MinusOneOrNotNegative (ChemicalParams->H);

  ChemicalParams->k_std_biodeg = NotNegative (ChemicalParams->k_std_biodeg);
  ChemicalParams->a_anaerobic = NotNegative (ChemicalParams->a_anaerobic);
  ChemicalParams->a_anoxic = NotNegative (ChemicalParams->a_anoxic);
  ChemicalParams->K_DO = NotNegative (ChemicalParams->K_DO);
  ChemicalParams->a_sorbed = NotNegative (ChemicalParams->a_sorbed);
  if ( ChemicalParams->Q10 < 1.0 )
    ChemicalParams->Q10 = 1.0;
  ChemicalParams->use_Monod = OneOrZero (ChemicalParams->use_Monod);
  ChemicalParams->k_max = NotNegative (ChemicalParams->k_max);
  ChemicalParams->Y = NotNegative (ChemicalParams->Y);
  ChemicalParams->K_s = NotNegative (ChemicalParams->K_s);
  ChemicalParams->b = NotNegative (ChemicalParams->b);

  ChemicalParams->k_instream = NotNegative (ChemicalParams->k_instream);

  ChemicalParams->k_vol = NotNegative (ChemicalParams->k_vol);
  ChemicalParams->k_deg = NotNegative (ChemicalParams->k_deg);

  ChemicalParams->m3_vol = OneOrZero (ChemicalParams->m3_vol);
  ChemicalParams->m3_sed = OneOrZero (ChemicalParams->m3_sed);
  ChemicalParams->m3_deg_bio = OneOrZero (ChemicalParams->m3_deg_bio);
  ChemicalParams->m3_deg_photo = OneOrZero (ChemicalParams->m3_deg_photo);
  ChemicalParams->m3_deg_hydr = OneOrZero (ChemicalParams->m3_deg_hydr);

  ChemicalParams->k_hydr_n = NotNegative (ChemicalParams->k_hydr_n);
  ChemicalParams->k_hydr_b = NotNegative (ChemicalParams->k_hydr_b);
  ChemicalParams->k_hydr_a = NotNegative (ChemicalParams->k_hydr_a);
  ChemicalParams->k_photo = NotNegative (ChemicalParams->k_photo);

  ChemicalParams->a_river = NotNegative (ChemicalParams->a_river);

  ChemicalParams->R_primary = NotNegative (ChemicalParams->R_primary);
  ChemicalParams->R_activated_sludge = NotNegative (ChemicalParams->R_activated_sludge);
  ChemicalParams->R_trickling_filter = NotNegative (ChemicalParams->R_trickling_filter);
  ChemicalParams->a_AS = NotNegative (ChemicalParams->a_AS);
  ChemicalParams->k_WWTP_nonbio = NotNegative (ChemicalParams->k_WWTP_nonbio);

  ChemicalParams->R_sewer = BetweenOneZero (ChemicalParams->R_sewer);
}

void CheckGeneralShot ( General_ParPtr gen_arg )
{
  /* nothing done here */
}

void CheckRiverShot ( River_ParPtr river_arg )
{
  river_arg->Q = AboveZero(river_arg->Q);
  river_arg->v = AboveZero(river_arg->v);
  river_arg->length = NotNegative(river_arg->length);
  river_arg->depth = NotNegative (river_arg->depth);
  river_arg->SS = MinusOneOrNotNegative (river_arg->SS);
  river_arg->DO = MinusOneOrNotNegative (river_arg->DO);
  river_arg->BOD = MinusOneOrNotNegative (river_arg->BOD);
  river_arg->pH = MinusOneOrNotNegative (river_arg->pH);
  river_arg->X = MinusOneOrNotNegative (river_arg->X);
}

void CheckDischargeShot ( FloatPtr shot_arg )
{
  int i;
  FloatType sum;

  DomesticEm_ParPtr dom =
    (DomesticEm_ParPtr)(&shot_arg[ ParametersArrayOffset[DOM] ]);

  NonDomesticEm_ParPtr nondom =
    (NonDomesticEm_ParPtr)(&shot_arg[ ParametersArrayOffset[NONDOM] ]);

  RunoffEm_ParPtr runoff =
    (RunoffEm_ParPtr)(&shot_arg[ ParametersArrayOffset[RUNOFF] ]);
/*
  OnSiteTreatment_ParPtr onsite =
    (OnSiteTreatment_ParPtr)(&shot_arg[ ParametersArrayOffset[ONSITE] ]);
*/
/*
  Sewer_ParPtr sewer =
    (Sewer_ParPtr)(&shot_arg[ ParametersArrayOffset[SEWER] ]);
*/
  Treatment_ParPtr treat =
    (Treatment_ParPtr)(&shot_arg[ ParametersArrayOffset[TREAT] ]);
/*
  SmallSurf_ParPtr surfwat =
    (SmallSurf_ParPtr)(shot_arg + ParametersArrayOffset[SURFWAT]);
*/

  /* domestic emission */

  dom->inhabitants = NotNegative( dom->inhabitants );
  dom->dry_weather_flow = NotNegative( dom->dry_weather_flow );

  /* nondomestic emission */

  for ( i=0 ; i<NR_SUB_NONDOM ; ++i )
    nondom->flow[i] = NotNegative( nondom->flow[i] );

  /* runoff */

  for ( i=0 ; i<NR_SUB_RUNOFF ; ++i )
    runoff->flow[i] = NotNegative( runoff->flow[i] );

  /* on-site-treatment */

  /* sewer */

  /* waste water treatment */

  sum = 0;
  
  for ( i=0 ; i<NR_WWTP_CLASSES ; ++i )
    /* treat->WWTP[i][0] */
    sum += ( treat->WWTP[i][1] = NotNegative(treat->WWTP[i][1]) );

  if ( sum == 0 ) 
    for ( i=0 ; i<NR_WWTP_CLASSES ; ++i )
      treat->WWTP[i][1] = 1.0 / (FloatType)NR_WWTP_CLASSES;

  else if ( sum != 1 )

    /* normalize to sum fractions = 1 */
    for ( i=0 ; i<NR_WWTP_CLASSES ; ++i )
      treat->WWTP[i][1] /= sum;

  /* small surface water */

  /* ground water */

  /*

     NOTE:

     Outflow fractions ARE NOT CHECKED here,
     because here we only have dest 1..end, not dest 0

     dest 0 = calculated as 1 - sum(other dest)
  */
}

void CheckEmissionShot ( FloatPtr shot_arg )
{
  EmissionParPtr em = (EmissionParPtr)shot_arg;

  /* em->emission_ID */
  em->consumption = NotNegative( em->consumption );
  em->nondom = NotNegative( em->nondom );
  em->runoff = NotNegative( em->runoff );
}

void CheckWWTPShot ( FloatPtr shot_arg )
{
  WWTP_ParPtr wwtp = (WWTP_ParPtr)shot_arg;
  PrimParPtr pr;
  ASParPtr as;
  TFParPtr tf;

  /* general */

  /*wwtp->WWTP_ID,*/
  /*wwtp->mode,*/

  switch ((int)wwtp->type)
  {
    case PRIMARY:

      pr = (PrimParPtr)shot_arg;

      pr->capacity_prim = MinusOneOrNotNegative (pr->capacity_prim);
      pr->h_prim = AboveZero (pr->h_prim);
      pr->v_overflow_prim = AboveZero (pr->v_overflow_prim);
      pr->R_SS_prim_max = AboveZero (pr->R_SS_prim_max);

      break;

    case ACTIVATED_SLUDGE:

      as = (ASParPtr)shot_arg;

      as->capacity_prim = MinusOneOrNotNegative (as->capacity_prim);
      as->h_prim = NotNegative (as->h_prim);
      as->v_overflow_prim = AboveZero (as->v_overflow_prim);
      as->R_SS_prim_max = AboveZero (as->R_SS_prim_max);

      as->h_sec = NotNegative (as->h_sec);
      as->v_overflow_sec = AboveZero (as->v_overflow_sec);
      as->R_SS_sec_max = AboveZero (as->R_SS_sec_max);

      as->capacity_sec = AboveZero (as->capacity_sec);
      as->nr_PE = MinusOneOrNotNegative (as->nr_PE);
      as->SS_ML = AboveZero (as->SS_ML);
      as->recycle = NotNegative (as->recycle);
      as->a_biomass = AboveZero (as->a_biomass);
      as->h_act_sludge = AboveZero (as->h_act_sludge);
      as->B_X = AboveZero (as->B_X);
      as->DO = AboveZero (as->DO);
      as->aer_type = OneOrZero (as->aer_type);
      as->f_aerobic = AboveZero (as->f_aerobic);
      as->f_anoxic = AboveZero (as->f_anoxic);
      as->f_anaerobic = AboveZero (as->f_anaerobic);
      as->eps_N = AboveZero (as->eps_N);
      as->eps_DN = AboveZero (as->eps_DN);

      break;
      
    case TRICKLING_FILTER:

      tf = (TFParPtr)shot_arg;
      /* mode 1, nothing to check ! */

      break;

    default:
      break;
  }
}

void CheckRivClassShot ( FloatPtr shot_arg )
{
  RivClass_ParPtr rc = (RivClass_ParPtr)shot_arg;

  /*RivClass_ID*/
  /*mode*/

  rc->lake = OneOrZero (rc->lake);

  rc->corr_riv_csewer = MinusOneToOne (rc->corr_riv_csewer);

  rc->SS = NotNegative (rc->SS);
  rc->k_SS = NotNegative (rc->k_SS);
  rc->f_oc = NotNegative (rc->f_oc);

  rc->v_sed_growth = NotNegative (rc->v_sed_growth);
  rc->eps_sed = NotNegative (rc->eps_sed);
  rc->ro_SS_sed = NotNegative (rc->ro_SS_sed);

  rc->X = NotNegative (rc->X);
  rc->DO = NotNegative (rc->DO);
  rc->BOD = NotNegative (rc->BOD);
  rc->pH = NotNegative (rc->pH);
  rc->a_rivclass = NotNegative (rc->a_rivclass);
  rc->k_light = NotNegative (rc->k_light);
}

void CheckCSewerQDWFShot ( FloatPtr shot_arg )
{
  if ( *shot_arg == -1 ) *shot_arg = 1;   /* in case no value was specified ! */
  *shot_arg = AboveZero (*shot_arg);
}

