/*
 * GREAT-ER
 *
 * SIMUL.C
 *
 * simulation startup section
 *
 * author =  Geert Boeije
 * state =   07.01.1999
*/

#include <time.h>
#include <stdio.h>
#include <string.h>
#include "init.h"
#include "simul.h"
#include "usriface.h"
#include "control.h"

#ifdef GUI_MODE

#include "extgui.h"

int WINAPI WinMain( HINSTANCE hInst,
                    HINSTANCE hPrevInstance, 
                    LPSTR lpszCmdParam, 
                    int nCmdShow )
{
  /* visibility options */

  /* defaults */

  Visible = 1;              /* default = visible operation */
  ShowMessages = 0;         /* default = no messages */
  ShowErrorMessages = 1;    /* default = show error messages */
  LogToFile = 0;            /* default = no log file */

  /* log file defaults */

  LogCtrlParams = 1;
  LogShotResults = 0;
  LogShotParameters = 0;
  LogSummaryResults = 1;

  if (lpszCmdParam)
    CheckCommandLine ( (char*)lpszCmdParam );

  if ( ExplanationWindow )
  {
    ShowExplanation ();
    return 0;
  }
  else
  {
    InitMainWindow ( hInst,
                     hPrevInstance, 
                     lpszCmdParam, 
                     nCmdShow );

    return Simul();
  }
}

#else

int main( int nr_arg, char** cmd_arg )
{
  int i;
  char cmd[500] = "";

  /* visibility options */

  /* defaults */

  Visible = 1;              /* default = visible operation */
  ShowMessages = 0;         /* default = no messages */
  ShowErrorMessages = 1;    /* default = show error messages */
  LogToFile = 0;            /* default = no log file */

  /* log file defaults */

  LogCtrlParams = 1;
  LogShotResults = 0;
  LogShotParameters = 0;
  LogSummaryResults = 1;

  if ( nr_arg > 1 )
    if (**(cmd_arg+1) != 10 )
    {
      for ( i=1 ; i<nr_arg ; ++i )
      {
        strcat ( cmd, *(cmd_arg + i) );
        strcat ( cmd, " " );
      }

      CheckCommandLine ( cmd );
    }

  if ( ExplanationWindow )
    printf ( EXPLANATION );
  else
    return Simul ();
}

#endif


/* command line options are:

   simul -x <window left x coord>         progress window location X
         -y <window top y coord>          progress window location Y
         -t "<window title>"              progress window title
         -d "<work directory>"            simulation directory name 
         -i "<INI file name>"             initialization file name
         -e "<error file name>"           error messages file name
         -s <nr Monte Carlo shots>        number of monte carlo samples
         -h                               switch to hide progress window
         -m                               switch to show control messages
         -w                               switch to hide warning messages
         -l                               switch to create a log file
         -?                               display command line options
*/


/* conversion to upper case */

char UpperSwitch ( char switch_arg )
{
  if ( 'a' <= switch_arg && 'z' >= switch_arg )
    return ( switch_arg+(int)'A'-(int)'a' );
  else
    return switch_arg;
}

/* a switch as such */
/* switch_arg must be lower case ! */

int A_Switch ( char* opt_arg, char switch_arg )
{
  int i;  
  int ok=0;
  char sw_upper = UpperSwitch ( switch_arg );

  for ( i=0 ; i<L ; ++i )
    if ( opt_arg[i] == '-' && ( opt_arg[i+1] == switch_arg || opt_arg[i+1] == sw_upper ) )
    {
      ok = 1;
      break;
    }
  return ok;
}

/* a switch plus a value */
/* switch_arg must be lower case ! */

int ValueSwitch  ( char* opt_arg, char switch_arg )
{
  int i,j,k;  
  char* sub_opt;
  int value = -1;
  char sw_upper = UpperSwitch ( switch_arg );

  for ( i=0 ; i<L ; ++i )
  {
    if ( opt_arg[i] == '-' && ( opt_arg[i+1] == switch_arg || opt_arg[i+1] == sw_upper ) )
    {
      sub_opt = &(opt_arg[i+2]);
      
      for ( j=0 ; j<L-i ; ++j )
      {
        k = 0;
        value = 0;

        while ( sub_opt[k] == ' ' )
          ++k;

        while ( sub_opt[k] >= '0' && sub_opt[k] <= '9' )
        {
          value = value * 10 + ((int)(sub_opt[k]) - (int)'0');
          ++k;
        }
      }
      break;
    }
  }  

  return value;
}



/* a switch plus a text string */
/* switch_arg must be lower case ! */

int TextSwitch ( char* opt_arg, char switch_arg, char* dest_arg )
{
  int i,j,k;  
  int ok = 0;
  char* sub_opt;
  char sw_upper = UpperSwitch ( switch_arg );

  for ( i=0 ; i<L ; ++i )
  {
    if ( opt_arg[i] == '-' && ( opt_arg[i+1] == switch_arg || opt_arg[i+1] == sw_upper ) )
    {
      sub_opt = &(opt_arg[i+2]);

      ok = 1;

      /* find first quote */
      
      for ( j=0 ; j<L-i ; ++j )
      {
        if ( sub_opt[j] == '"' )
        {
          k = 0;
          sub_opt += j;

          /* until the second quote */

          while ( sub_opt[k+1] != '"' && sub_opt[k] != '\0' )
          {
            dest_arg[k] = sub_opt[k+1];
            ++k;
          }
          
          dest_arg[k] = '\0';
          
          break;
        }
      }
      break;
    }
  }

  return ok;
}

/* check the command line ! */

void CheckCommandLine ( char* opt_arg )
{
  int i;
  int itemp;
  char ctemp[500];

  L = strlen ( opt_arg );

  #ifdef GUI_MODE

  /* find -x */
  /* upper left X coordinate */  

  if ( ( itemp = ValueSwitch  ( opt_arg, 'x' )) != -1 )
    Window_X = itemp;

  /* find -y */
  /* upper left Y coordinate */

  if ( ( itemp = ValueSwitch  ( opt_arg, 'y' )) != -1 )
    Window_Y = itemp;

  /* find -t */
  /* simulation title */

  if ( TextSwitch ( opt_arg, 't', ctemp  ) )
    strcpy (SimulationTitle, ctemp);

  #endif

  /* find -s */
  /* # Monte Carlo Shots */

  if ( ( itemp = ValueSwitch  ( opt_arg, 's' )) != -1 )
    NrMonteCarloShots = itemp;
  
  if ( NrMonteCarloShots < 1 )
    NrMonteCarloShots = 1;

  /* find -i */
  /* INI file name */
  
  if ( TextSwitch ( opt_arg, 'i', ctemp ) )
    strcpy (INIFileName, ctemp);

  /* find -e */
  /* ERROR file name */
  
  if ( TextSwitch ( opt_arg, 'e', ctemp ) )
    strcpy (ErrorFileName, ctemp);

  /* find -d */
  /* directory */
  
  if ( TextSwitch ( opt_arg, 'd', ctemp  ) )
    strcpy (WorkDirectory, ctemp);



  #ifdef GUI_MODE

  /* find -H */
  /* hide simulation progress window */

  if ( A_Switch ( opt_arg, 'h' ) )
    Visible = 0;

  /* find -M */
  /* show simulation control messages */
  
  if ( A_Switch ( opt_arg, 'm' ) )
    if ( Visible )
      ShowMessages = 1;

  /* find -W */
  /* don't show simulation error messages */

  if ( A_Switch ( opt_arg, 'w' ) )
    ShowErrorMessages = 0;

  #endif

  /* find -L */
  /* create a log file */
  /* note that an error log file is ALWAYS created when errors occur */

  if ( A_Switch ( opt_arg, 'l' ) )
    LogToFile = 1;

  /* find -? */
  /* display command line options */

  for ( i=0 ; i<L ; ++i )
    if ( opt_arg[i] == '-' && opt_arg[i+1] == '?' )
    {
      ExplanationWindow = 1;
      break;
    }
}

